@@ -20,7 +20,9 @@ OVN_SG_EXT_ID_KEY = 'neutron:security_group_id'
 OVN_SG_RULE_EXT_ID_KEY = 'neutron:security_group_rule_id'
 OVN_ML2_MECH_DRIVER_NAME = 'ovn'
 OVN_NETWORK_NAME_EXT_ID_KEY = 'neutron:network_name'
+OVN_NETWORK_MTU_EXT_ID_KEY = 'neutron:mtu'
 OVN_PORT_NAME_EXT_ID_KEY = 'neutron:port_name'
+OVN_PORT_FIP_EXT_ID_KEY = 'neutron:port_fip'
 OVN_ROUTER_NAME_EXT_ID_KEY = 'neutron:router_name'
 OVN_ROUTER_IS_EXT_GW = 'neutron:is_ext_gw'
 OVN_GW_PORT_EXT_ID_KEY = 'neutron:gw_port_id'
@@ -45,8 +47,10 @@ OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': six.string_types,
                                     'tag': six.integer_types},
                                    {'vtep-physical-switch': six.string_types,
                                     'vtep-logical-switch': six.string_types}]
+MIGRATING_ATTR = 'migrating_to'
 OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses']
 OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
+OVN_CHASSIS_REDIRECT = 'chassisredirect'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
 OVN_DROP_PORT_GROUP_NAME = 'neutron_pg_drop'
 
@@ -150,3 +154,15 @@ MAINTENANCE_DELETE_TYPE_ORDER = {
 # The addresses field to set in the logical switch port which has a
 # peer router port (connecting to the logical router).
 DEFAULT_ADDR_FOR_LSP_WITH_PEER = 'router'
+
+# Loadbalancer constants
+LRP_PREFIX = "lrp-"
+LB_VIP_PORT_PREFIX = "ovn-lb-vip-"
+
+# Maximum chassis count where a gateway port can be hosted
+MAX_GW_CHASSIS = 5
+
+UNKNOWN_ADDR = 'unknown'
+
+# TODO(lucasagomes): Create constants for other LSP types
+LSP_TYPE_LOCALNET = 'localnet'
