@@ -204,7 +204,7 @@ class OVNClient(object):
             port_type = 'vtep'
             options = {'vtep-physical-switch': vtep_physical_switch,
                        'vtep-logical-switch': vtep_logical_switch}
-            addresses = ["unknown"]
+            addresses = [ovn_const.UNKNOWN_ADDR]
             parent_name = []
             tag = []
             port_security = []
@@ -227,19 +227,21 @@ class OVNClient(object):
             addresses = [address]
             addresses.extend(new_macs)
 
-            if not port_security:
+            # Only adjust the OVN type if the port is not owned by Neutron
+            # DHCP agents.
+            if (port['device_owner'] == const.DEVICE_OWNER_DHCP and
+                    not utils.is_neutron_dhcp_agent_port(port)):
+                port_type = 'localport'
+
+            # The "unknown" address should only be set for the normal LSP
+            # ports (the ones which type is empty)
+            if not port_security and not port_type:
                 # Port security is disabled for this port.
                 # So this port can send traffic with any mac address.
                 # OVN allows any mac address from a port if "unknown"
                 # is added to the Logical_Switch_Port.addresses column.
                 # So add it.
-                addresses.append("unknown")
-
-            # Only adjust the OVN type if the port is not owned by Neutron
-            # DHCP agents.
-            if (port['device_owner'] == const.DEVICE_OWNER_DHCP and
-                    not port['device_id'].startswith('dhcp')):
-                port_type = 'localport'
+                addresses.append(ovn_const.UNKNOWN_ADDR)
 
         dhcpv4_options = self._get_port_dhcp_options(port, const.IP_VERSION_4)
         dhcpv6_options = self._get_port_dhcp_options(port, const.IP_VERSION_6)
@@ -300,19 +302,33 @@ class OVNClient(object):
             # The lport_name *must* be neutron port['id'].  It must match the
             # iface-id set in the Interfaces table of the Open_vSwitch
             # database which nova sets to be the port ID.
+
+            kwargs = {
+                'lport_name': port['id'],
+                'lswitch_name': lswitch_name,
+                'addresses': port_info.addresses,
+                'external_ids': external_ids,
+                'parent_name': port_info.parent_name,
+                'tag': port_info.tag,
+                'enabled': port.get('admin_state_up'),
+                'options': port_info.options,
+                'type': port_info.type,
+                'port_security': port_info.port_security,
+                'dhcpv4_options': dhcpv4_options,
+                'dhcpv6_options': dhcpv6_options
+            }
+
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                kwargs['addresses'] = []
+
             port_cmd = txn.add(self._nb_idl.create_lswitch_port(
-                lport_name=port['id'],
-                lswitch_name=lswitch_name,
-                addresses=port_info.addresses,
-                external_ids=external_ids,
-                parent_name=port_info.parent_name,
-                tag=port_info.tag,
-                enabled=port.get('admin_state_up'),
-                options=port_info.options,
-                type=port_info.type,
-                port_security=port_info.port_security,
-                dhcpv4_options=dhcpv4_options,
-                dhcpv6_options=dhcpv6_options))
+                **kwargs))
 
             # Handle ACL's for this port. If we're not using Port Groups
             # because either the schema doesn't support it or we didn't
@@ -417,6 +433,16 @@ class OVNClient(object):
                 dhcpv6_options = txn.add(port_info.dhcpv6_options['cmd'])
             else:
                 dhcpv6_options = [port_info.dhcpv6_options['uuid']]
+
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                columns_dict['addresses'] = []
+
             # NOTE(lizk): Fail port updating if port doesn't exist. This
             # prevents any new inserted resources to be orphan, such as port
             # dhcp options or ACL rules for port, e.g. a port was created
@@ -601,9 +627,7 @@ class OVNClient(object):
         context = n_context.get_admin_context()
         fip_db = self._l3_plugin._get_floatingip(context, floatingip['id'])
 
-        func = self._nb_idl.add_nat_rule_in_lrouter
         gw_lrouter_name = utils.ovn_name(router_id)
-        nat_rule_args = (gw_lrouter_name,)
         # TODO(chandrav): Since the floating ip port is not
         # bound to any chassis, packets destined to floating ip
         # will be dropped. To overcome this, delete the floating
@@ -614,21 +638,6 @@ class OVNClient(object):
                         fip_db['floating_port_id'],
                         utils.ovn_name(floatingip['floating_network_id'])))
 
-        # Get the list of nat rules and check if the external_ip
-        # with type 'dnat_and_snat' already exists or not.
-        # If exists, set the new value.
-        # This happens when the port associated to a floating ip
-        # is deleted before the disassociation.
-        lrouter_nat_rules = self._nb_idl.get_lrouter_nat_rules(
-            gw_lrouter_name)
-        for nat_rule in lrouter_nat_rules:
-            if (nat_rule['external_ip'] ==
-                    floatingip['floating_ip_address'] and
-                    nat_rule['type'] == 'dnat_and_snat'):
-                func = self._nb_idl.set_nat_rule_in_lrouter
-                nat_rule_args = (gw_lrouter_name, nat_rule['uuid'])
-                break
-
         ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
@@ -645,14 +654,36 @@ class OVNClient(object):
             columns['logical_port'] = floatingip['port_id']
             ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
             if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
-                columns['external_mac'] = port['mac_address']
+                port_net = self._plugin.get_network(context,
+                                                    port['network_id'])
+                if port_net.get(pnet.NETWORK_TYPE) != const.TYPE_VLAN:
+                    columns['external_mac'] = port['mac_address']
 
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        commands.append(func(*nat_rule_args, **columns))
+        commands.append(self._nb_idl.add_nat_rule_in_lrouter(gw_lrouter_name,
+                                                             **columns))
+
+        # Get the logical port (of the private network) and set the field
+        # external_ids:fip=<FIP>. This will be used by the ovn octavia driver
+        # to add the floating ip as vip in the Load_Balancer.vips column.
+        private_lsp = self._nb_idl.get_lswitch_port(floatingip['port_id'])
+
+        if private_lsp:
+            port_fip = {
+                ovn_const.OVN_PORT_FIP_EXT_ID_KEY:
+                    floatingip['floating_ip_address']}
+            commands.append(
+                self._nb_idl.db_set('Logical_Switch_Port', private_lsp.uuid,
+                                    ('external_ids', port_fip))
+            )
+        else:
+            LOG.warning("LSP for floatingip %s, has not been found! "
+                        "Cannot set FIP on VIP.",
+                        floatingip['id'])
         self._transaction(commands, txn=txn)
 
     def _delete_floatingip(self, fip, lrouter, txn=None):
@@ -660,6 +691,20 @@ class OVNClient(object):
                     lrouter, type='dnat_and_snat',
                     logical_ip=fip['logical_ip'],
                     external_ip=fip['external_ip'])]
+        try:
+            port_id = (
+                fip['external_ids'].get(ovn_const.OVN_FIP_PORT_EXT_ID_KEY))
+            if port_id:
+                private_lsp = self._nb_idl.get_lswitch_port(port_id)
+                if private_lsp:
+                    commands.append(
+                        self._nb_idl.db_remove(
+                            'Logical_Switch_Port', private_lsp.uuid,
+                            'external_ids',
+                            (ovn_const.OVN_PORT_FIP_EXT_ID_KEY))
+                    )
+        except KeyError:
+            LOG.debug("FIP %s doesn't have external_ids.", fip)
         self._transaction(commands, txn=txn)
 
     def update_floatingip_status(self, floatingip):
@@ -710,15 +755,10 @@ class OVNClient(object):
             floatingip['id'], floatingip, ovn_const.TYPE_FLOATINGIPS)
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
-            if (ovn_fip and
-                (floatingip['fixed_ip_address'] != ovn_fip['logical_ip'] or
-                 floatingip['port_id'] != ovn_fip['external_ids'].get(
-                    ovn_const.OVN_FIP_PORT_EXT_ID_KEY))):
-
+            if ovn_fip:
                 lrouter = ovn_fip['external_ids'].get(
                     ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY,
                     utils.ovn_name(router_id))
-
                 self._delete_floatingip(ovn_fip, lrouter, txn=txn)
                 fip_status = const.FLOATINGIP_STATUS_DOWN
 
@@ -1087,7 +1127,7 @@ class OVNClient(object):
         candidates = cms_bmaps or bmaps
         if not cms_bmaps:
             LOG.debug("No eligible chassis with external connectivity"
-                      " through ovn-cms-options.")
+                      " through ovn-cms-options for %s", physnet)
         LOG.debug("Chassis candidates with external connectivity: %s",
                   candidates)
         return candidates
@@ -1102,7 +1142,9 @@ class OVNClient(object):
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
                 port, ovn_const.TYPE_ROUTER_PORTS)),
             ovn_const.OVN_SUBNET_EXT_IDS_KEY:
-                ' '.join(utils.get_port_subnet_ids(port))}
+                ' '.join(utils.get_port_subnet_ids(port)),
+            ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY:
+                utils.ovn_name(port['network_id'])}
 
         router_id = port.get('device_id')
         if router_id:
@@ -1317,15 +1359,16 @@ class OVNClient(object):
         txn.add(self._nb_idl.create_lswitch_port(
             lport_name=utils.ovn_provnet_port_name(network['id']),
             lswitch_name=utils.ovn_name(network['id']),
-            addresses=['unknown'],
+            addresses=[ovn_const.UNKNOWN_ADDR],
             external_ids={},
-            type='localnet',
+            type=ovn_const.LSP_TYPE_LOCALNET,
             tag=tag if tag else [],
             options={'network_name': physnet}))
 
     def _gen_network_external_ids(self, network):
         ext_ids = {
             ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY: network['name'],
+            ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY: str(network['mtu']),
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(
                 utils.get_revision_number(network, ovn_const.TYPE_NETWORKS))}
 
@@ -1418,7 +1461,19 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
             ext_ids = self._gen_network_external_ids(network)
+            lswitch = self._nb_idl.get_lswitch(lswitch_name)
             txn.add(self._nb_idl.set_lswitch_ext_ids(lswitch_name, ext_ids))
+            # Check if previous mtu is different than current one,
+            # checking will help reduce number of operations
+            if (not lswitch or
+                    lswitch.external_ids.get(
+                        ovn_const.OVN_NETWORK_MTU_EXT_ID_KEY) !=
+                    str(network['mtu'])):
+                context = n_context.get_admin_context()
+                subnets = self._plugin.get_subnets_by_network(
+                    context, network['id'])
+                for subnet in subnets:
+                    self.update_subnet(subnet, network, txn)
 
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             if qos_update_required:
@@ -1500,6 +1555,12 @@ class OVNClient(object):
             'mtu': str(mtu),
         }
 
+        net_dns_domain = network.get('dns_domain', '').rstrip('.')
+        if net_dns_domain:
+            # NOTE(mjozefcz): String field should be with quotes,
+            # otherwise ovn will try to resolve it as variable.
+            options['domain_name'] = '"%s"' % net_dns_domain
+
         if subnet['gateway_ip']:
             options['router'] = subnet['gateway_ip']
 
@@ -1667,7 +1728,15 @@ class OVNClient(object):
             self._add_subnet_dhcp_options(subnet, network)
         db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
-    def update_subnet(self, subnet, network):
+    def _modify_subnet_dhcp_options(self, subnet, ovn_subnet, network, txn):
+        if subnet['enable_dhcp'] and not ovn_subnet:
+            self._enable_subnet_dhcp_options(subnet, network, txn)
+        elif subnet['enable_dhcp'] and ovn_subnet:
+            self._update_subnet_dhcp_options(subnet, network, txn)
+        elif not subnet['enable_dhcp'] and ovn_subnet:
+            self._remove_subnet_dhcp_options(subnet['id'], txn)
+
+    def update_subnet(self, subnet, network, txn=None):
         ovn_subnet = self._nb_idl.get_subnet_dhcp_options(
             subnet['id'])['subnet']
 
@@ -1677,15 +1746,13 @@ class OVNClient(object):
 
         check_rev_cmd = self._nb_idl.check_revision_number(
             subnet['id'], subnet, ovn_const.TYPE_SUBNETS)
-        with self._nb_idl.transaction(check_error=True) as txn:
-            txn.add(check_rev_cmd)
-            if subnet['enable_dhcp'] and not ovn_subnet:
-                self._enable_subnet_dhcp_options(subnet, network, txn)
-            elif not subnet['enable_dhcp'] and ovn_subnet:
-                self._remove_subnet_dhcp_options(subnet['id'], txn)
-            elif subnet['enable_dhcp'] and ovn_subnet:
-                self._update_subnet_dhcp_options(subnet, network, txn)
-
+        if not txn:
+            with self._nb_idl.transaction(check_error=True) as txn_n:
+                txn_n.add(check_rev_cmd)
+                self._modify_subnet_dhcp_options(subnet, ovn_subnet, network,
+                                                 txn_n)
+        else:
+            self._modify_subnet_dhcp_options(subnet, ovn_subnet, network, txn)
         if check_rev_cmd.result == ovn_const.TXN_COMMITTED:
             db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
@@ -1722,11 +1789,11 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             if not self._nb_idl.get_port_group(pg_name):
                 # If drop Port Group doesn't exist yet, create it.
-                txn.add(self._nb_idl.pg_add(pg_name, acls=[]))
+                txn.add(self._nb_idl.pg_add(pg_name, acls=[], may_exist=True))
                 # Add ACLs to this Port Group so that all traffic is dropped.
                 acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
                 for acl in acls:
-                    txn.add(self._nb_idl.pg_acl_add(**acl))
+                    txn.add(self._nb_idl.pg_acl_add(may_exist=True, **acl))
 
             if ports:
                 ports_ids = [port['id'] for port in ports]
@@ -1777,10 +1844,10 @@ class OVNClient(object):
         ports = self._plugin.get_ports(context, filters=dict(
             network_id=[network_id], device_owner=[const.DEVICE_OWNER_DHCP]))
 
-        # Metadata ports are DHCP ports without a device_id starting by 'dhcp'
-        # since those belong to Neutron DHCP agents.
+        # Metadata ports are DHCP ports not belonging to the Neutron
+        # DHCP agents
         for port in ports:
-            if not port['device_id'].startswith('dhcp'):
+            if not utils.is_neutron_dhcp_agent_port(port):
                 return port
 
     def _find_metadata_port_ip(self, context, subnet):
@@ -1917,7 +1984,7 @@ class OVNClient(object):
             for old_hostname, old_ips in old_records.items():
                 if records_to_add.get(old_hostname) != old_ips:
                     txn.add(self._nb_idl.dns_remove_record(
-                        ls_dns_record.uuid, old_hostname))
+                        ls_dns_record.uuid, old_hostname, if_exists=True))
 
         for hostname, ips in records_to_add.items():
             if ls_dns_record.records.get(hostname) != ips:
@@ -1950,4 +2017,4 @@ class OVNClient(object):
         for hostname in hostnames:
             if ls_dns_record.records.get(hostname):
                 txn.add(self._nb_idl.dns_remove_record(
-                        ls_dns_record.uuid, hostname))
+                        ls_dns_record.uuid, hostname, if_exists=True))
