@@ -27,7 +27,9 @@ from neutron_lib.plugins import directory
 from neutron_lib.utils import net as n_utils
 from oslo_utils import netutils
 from oslo_utils import strutils
-
+from ovs.db import idl
+from ovsdbapp.backend.ovs_idl import connection
+from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn._i18n import _
 from networking_ovn.common import constants
@@ -53,7 +55,7 @@ def ovn_lrouter_port_name(id):
     #   - patch-lrp-<UUID>-to-<UUID>
     #   - patch-<UUID>-to-lrp-<UUID>
     # lrp stands for Logical Router Port
-    return 'lrp-%s' % id
+    return constants.LRP_PREFIX + '%s' % id
 
 
 def ovn_provnet_port_name(network_id):
@@ -368,9 +370,14 @@ def get_port_subnet_ids(port):
     return [f['subnet_id'] for f in fixed_ips]
 
 
-def ovn_metadata_name(id_):
-    """Return the OVN metadata name based on an id."""
-    return 'metadata-%s' % id_
+def get_ovsdb_connection(connection_string, schema, timeout, tables=None):
+    helper = idlutils.get_schema_helper(connection_string, schema)
+    if tables:
+        for table in tables:
+            helper.register_table(table)
+    else:
+        helper.register_all()
+    return connection.Connection(idl.Idl(connection_string, helper), timeout)
 
 
 def get_method_class(method):
@@ -379,6 +386,11 @@ def get_method_class(method):
     return method.__self__.__class__
 
 
+def ovn_metadata_name(id_):
+    """Return the OVN metadata name based on an id."""
+    return 'metadata-%s' % id_
+
+
 def is_gateway_chassis_invalid(chassis_name, gw_chassis,
                                physnet, chassis_physnets):
     """Check if gateway chassis is invalid
