@@ -12,6 +12,7 @@
 #    under the License.
 #
 
+import copy
 import functools
 import operator
 import threading
@@ -484,7 +485,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         drastically affect performance.  Raising an exception will
         result in the deletion of the resource.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
         self._ovn_client.create_port(port)
         self._notify_dhcp_updated(port['id'])
@@ -538,10 +539,30 @@ class OVNMechanismDriver(api.MechanismDriver):
         state. It is up to the mechanism driver to ignore state or
         state changes that it does not know or care about.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
-        original_port = context.original
+        original_port = copy.deepcopy(context.original)
         original_port['network'] = context.network.current
+
+        # NOTE(mjozefcz): Check if port is in migration state. If so update
+        # the port status from DOWN to UP in order to generate 'fake'
+        # vif-interface-plugged event. This workaround is needed to
+        # perform live-migration with live_migration_wait_for_vif_plug=True.
+        if ((port['status'] == const.PORT_STATUS_DOWN and
+             ovn_const.MIGRATING_ATTR in port[portbindings.PROFILE].keys() and
+             port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_OVS)):
+            admin_context = n_context.get_admin_context()
+            LOG.info("Setting port %s status from DOWN to UP in order "
+                     "to emit vif-interface-plugged event.",
+                     port['id'])
+            self._plugin.update_port_status(admin_context, port['id'],
+                                            const.PORT_STATUS_ACTIVE)
+            # The revision has been changed. In the meantime
+            # port-update event already updated the OVN configuration,
+            # So there is no need to update it again here. Anyway it
+            # will fail that OVN has port with bigger revision.
+            return
+
         self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
@@ -557,7 +578,7 @@ class OVNMechanismDriver(api.MechanismDriver):
         expected, and will not prevent the resource from being
         deleted.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
         port['network'] = context.network.current
         self._ovn_client.delete_port(port['id'], port_object=port)
 
@@ -698,19 +719,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         # See doc/source/design/ovn_worker.rst for more details.
         return [worker.OvnWorker(), maintenance.MaintenanceWorker()]
 
-    def _update_subport_host_if_needed(self, port_id):
-        parent_port = self._ovn_client.get_parent_port(port_id)
-        if parent_port:
-            admin_context = n_context.get_admin_context()
-            try:
-                port = self._plugin.get_port(admin_context, parent_port)
-                host_id = port.get(portbindings.HOST_ID, '')
-                subport = {'port': {'binding:host_id': host_id}}
-                self._plugin.update_port(admin_context, port_id, subport)
-            except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
-                LOG.debug("Error trying to set host_id %s for subport %s",
-                          host_id, port_id)
-
     def _update_dnat_entry_if_needed(self, port_id, up=True):
         """Update DNAT entry if using distributed floating ips."""
         if not config.is_ovn_distributed_floating_ip():
@@ -747,6 +755,15 @@ class OVNMechanismDriver(api.MechanismDriver):
             self._nb_ovn.db_clear(
                 'NAT', nat['_uuid'], 'external_mac').execute(check_error=True)
 
+    def _should_notify_nova(self, db_port):
+        # NOTE(twilson) It is possible for a test to override a config option
+        # after the plugin has been initialized so the nova_notifier attribute
+        # is not set on the plugin
+        return (cfg.CONF.notify_nova_on_port_status_changes and
+                hasattr(self._plugin, 'nova_notifier') and
+                db_port.device_owner.startswith(
+                    const.DEVICE_OWNER_COMPUTE_PREFIX))
+
     def set_port_status_up(self, port_id):
         # Port provisioning is complete now that OVN has reported that the
         # port is up. Any provisioning block (possibly added during port
@@ -756,11 +773,6 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id)
         self._wait_for_metadata_provisioned_if_needed(port_id)
 
-        # If this port is a subport, we need to update the host_id and set it
-        # to its parent's. Otherwise, Neutron won't even try to bind it and
-        # it will not transition from DOWN to ACTIVE.
-        self._update_subport_host_if_needed(port_id)
-
         provisioning_blocks.provisioning_complete(
             n_context.get_admin_context(),
             port_id,
@@ -784,8 +796,7 @@ class OVNMechanismDriver(api.MechanismDriver):
                                         const.DEVICE_OWNER_ROUTER_HA_INTF):
                 self._plugin.update_port_status(admin_context, port_id,
                                                 const.PORT_STATUS_ACTIVE)
-            elif db_port.device_owner.startswith(
-                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+            elif self._should_notify_nova(db_port):
                 self._plugin.nova_notifier.notify_port_active_direct(db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug('Port not found during OVN status up report: %s',
@@ -809,8 +820,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             self._plugin.update_port_status(admin_context, port_id,
                                             const.PORT_STATUS_DOWN)
 
-            if db_port.device_owner.startswith(
-                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+            if self._should_notify_nova(db_port):
                 self._plugin.nova_notifier.record_port_status_changed(
                     db_port, const.PORT_STATUS_ACTIVE, const.PORT_STATUS_DOWN,
                     None)
@@ -858,8 +868,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         """Wait for metadata service to be provisioned.
 
         Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled, this function will return right
-        away.
+        it resides. If metadata is disabled or DHCP is not enabled for its
+        subnets, this function will return right away.
         """
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
@@ -874,6 +884,26 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Logical port %s is not bound to a "
                             "chassis", port_id)
                 return
+
+            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
+            context = n_context.get_admin_context()
+            port = self._plugin.get_port(context, port_id)
+            port_subnet_ids = set(
+                ip['subnet_id'] for ip in port['fixed_ips'] if
+                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
+            if not port_subnet_ids:
+                # The port doesn't belong to any IPv4 subnet
+                return
+
+            subnets = self._plugin.get_subnets(context, filters=dict(
+                network_id=[port['network_id']], ip_version=[4],
+                enable_dhcp=True))
+
+            subnet_ids = set(
+                s['id'] for s in subnets if s['id'] in port_subnet_ids)
+            if not subnet_ids:
+                return
+
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
