@@ -144,7 +144,7 @@ def get_connection(db_class, trigger=None, driver=None):
         args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
         cls = ovsdb_monitor.OvnSbIdl
 
-    if trigger and utils.get_method_class(trigger) == ovsdb_monitor.OvnWorker:
+    if trigger and utils.get_method_class(trigger).__name__ == 'OvnWorker':
         idl_ = cls.from_server(*args, driver=driver)
     else:
         if db_class == OvsdbSbOvnIdl:
@@ -575,12 +575,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return cmd.SetNATRuleInLRouterCommand(self, lrouter, nat_rule_uuid,
                                               **columns)
 
-    def add_nat_ip_to_lrport_peer_options(self, lport, nat_ip):
-        return cmd.AddNatIpToLRPortPeerOptionsCommand(self, lport, nat_ip)
-
-    def delete_nat_ip_from_lrport_peer_options(self, lport, nat_ip):
-        return cmd.DeleteNatIpFromLRPortPeerOptionsCommand(self, lport, nat_ip)
-
     def get_lswitch_port(self, lsp_name):
         try:
             return self.lookup('Logical_Switch_Port', lsp_name)
@@ -818,7 +812,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def set_port_cidrs(self, name, cidrs):
         # TODO(twilson) add if_exists to db commands
         return self.db_set('Port_Binding', name, 'external_ids',
-                           {'neutron-port-cidrs': cidrs}, if_exists=True)
+                           {'neutron-port-cidrs': cidrs})
 
     def get_ports_on_chassis(self, chassis):
         # TODO(twilson) Some day it would be nice to stop passing names around
