@@ -15,10 +15,12 @@
 
 import mock
 
+from futurist import periodics
 from neutron.tests.unit.api import test_extensions
 from neutron.tests.unit.extensions import test_extraroute
 from neutron.tests.unit.extensions import test_securitygroup
 
+from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import maintenance
 from networking_ovn.common import utils
@@ -97,13 +99,38 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                     ovn_const.OVN_PORT_NAME_EXT_ID_KEY) == name):
                 return row
 
-    def _create_subnet(self, name, net_id):
+    def _set_global_dhcp_opts(self, ip_version, opts):
+        opt_string = ','.join(['{0}:{1}'.format(key, value)
+                               for key, value
+                               in opts.items()])
+        if ip_version == 6:
+            ovn_config.cfg.CONF.set_override('ovn_dhcp6_global_options',
+                                             opt_string,
+                                             group='ovn')
+        if ip_version == 4:
+            ovn_config.cfg.CONF.set_override('ovn_dhcp4_global_options',
+                                             opt_string,
+                                             group='ovn')
+
+    def _unset_global_dhcp_opts(self, ip_version):
+        if ip_version == 6:
+            ovn_config.cfg.CONF.clear_override('ovn_dhcp6_global_options',
+                                               group='ovn')
+        if ip_version == 4:
+            ovn_config.cfg.CONF.clear_override('ovn_dhcp4_global_options',
+                                               group='ovn')
+
+    def _create_subnet(self, name, net_id, ip_version=4):
         data = {'subnet': {'name': name,
                            'tenant_id': self._tenant_id,
                            'network_id': net_id,
-                           'cidr': '10.0.0.0/24',
-                           'ip_version': 4,
+                           'ip_version': ip_version,
                            'enable_dhcp': True}}
+        if ip_version == 4:
+            data['subnet']['cidr'] = '10.0.0.0/24'
+        else:
+            data['subnet']['cidr'] = 'eef0::/64'
+
         req = self.new_create_request('subnets', data, self.fmt)
         res = req.get_response(self.api)
         return self.deserialize(self.fmt, res)['subnet']
@@ -196,6 +223,7 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
                 return row
 
 
+@mock.patch('networking_ovn.db.maintenance.INCONSISTENCIES_OLDER_THAN', -1)
 class TestMaintenance(_TestMaintenanceHelper):
 
     def test_network(self):
@@ -325,6 +353,108 @@ class TestMaintenance(_TestMaintenanceHelper):
         # Assert the revision number no longer exists
         self.assertIsNone(db_rev.get_revision_row(neutron_obj['id']))
 
+    def test_subnet_global_dhcp4_opts(self):
+        obj_name = 'globaltestsubnet'
+        options = {'ntp_server': '1.2.3.4'}
+        neutron_net = self._create_network('network1')
+
+        # Create a subnet without global options
+        neutron_sub = self._create_subnet(obj_name, neutron_net['id'])
+
+        # Assert that the option is not set
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertIsNone(ovn_obj.options.get('ntp_server', None))
+
+        # Set some global DHCP Options
+        self._set_global_dhcp_opts(ip_version=4, opts=options)
+
+        # Run the maintenance task to add the new options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was added
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertEqual(
+            ovn_obj.options.get('ntp_server', None),
+            '1.2.3.4')
+
+        # Change the global option
+        new_options = {'ntp_server': '4.3.2.1'}
+        self._set_global_dhcp_opts(ip_version=4, opts=new_options)
+
+        # Run the maintenance task to update the options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was changed
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertEqual(
+            ovn_obj.options.get('ntp_server', None),
+            '4.3.2.1')
+
+        # Change the global option to null
+        new_options = {'ntp_server': ''}
+        self._set_global_dhcp_opts(ip_version=4, opts=new_options)
+
+        # Run the maintenance task to update the options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was removed
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertIsNone(ovn_obj.options.get('ntp_server', None))
+
+    def test_subnet_global_dhcp6_opts(self):
+        obj_name = 'globaltestsubnet'
+        options = {'ntp_server': '1.2.3.4'}
+        neutron_net = self._create_network('network1')
+
+        # Create a subnet without global options
+        neutron_sub = self._create_subnet(obj_name, neutron_net['id'], 6)
+
+        # Assert that the option is not set
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertIsNone(ovn_obj.options.get('ntp_server', None))
+
+        # Set some global DHCP Options
+        self._set_global_dhcp_opts(ip_version=6, opts=options)
+
+        # Run the maintenance task to add the new options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was added
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertEqual(
+            ovn_obj.options.get('ntp_server', None),
+            '1.2.3.4')
+
+        # Change the global option
+        new_options = {'ntp_server': '4.3.2.1'}
+        self._set_global_dhcp_opts(ip_version=6, opts=new_options)
+
+        # Run the maintenance task to update the options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was changed
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertEqual(
+            ovn_obj.options.get('ntp_server', None),
+            '4.3.2.1')
+
+        # Change the global option to null
+        new_options = {'ntp_server': ''}
+        self._set_global_dhcp_opts(ip_version=6, opts=new_options)
+
+        # Run the maintenance task to update the options
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_global_dhcp_opts)
+
+        # Assert that the option was removed
+        ovn_obj = self._find_subnet_row_by_id(neutron_sub['id'])
+        self.assertIsNone(ovn_obj.options.get('ntp_server', None))
+
     def test_subnet(self):
         obj_name = 'subnettest'
         neutron_net = self._create_network('network1')
@@ -565,3 +695,102 @@ class TestMaintenance(_TestMaintenanceHelper):
 
         # Assert the revision number no longer exists
         self.assertIsNone(db_rev.get_revision_row(neutron_obj['port_id']))
+
+    def test_check_metadata_ports(self):
+        ovn_config.cfg.CONF.set_override('ovn_metadata_enabled', True,
+                                         group='ovn')
+        neutron_net = self._create_network('network1')
+        metadata_port = self._ovn_client._find_metadata_port(
+            self.context, neutron_net['id'])
+
+        # Assert the metadata port exists
+        self.assertIsNotNone(metadata_port)
+
+        # Delete the metadata port
+        self._delete('ports', metadata_port['id'])
+
+        metadata_port = self._ovn_client._find_metadata_port(
+            self.context, neutron_net['id'])
+
+        # Assert the metadata port is gone
+        self.assertIsNone(metadata_port)
+
+        # Call the maintenance thread to fix the problem, it will raise
+        # NeverAgain so that the job only runs once at startup
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_metadata_ports)
+
+        metadata_port = self._ovn_client._find_metadata_port(
+            self.context, neutron_net['id'])
+
+        # Assert the metadata port was re-created
+        self.assertIsNotNone(metadata_port)
+
+    def test_check_metadata_ports_not_enabled(self):
+        ovn_config.cfg.CONF.set_override('ovn_metadata_enabled', False,
+                                         group='ovn')
+        with mock.patch.object(self._ovn_client,
+                               'create_metadata_port') as mock_create_port:
+            self.assertRaises(periodics.NeverAgain,
+                              self.maint.check_metadata_ports)
+            # Assert create_metadata_port() wasn't called since metadata
+            # is not enabled
+            self.assertFalse(mock_create_port.called)
+
+    def test_check_for_port_security_unknown_address(self):
+        neutron_net = self._create_network('network1')
+        neutron_port = self._create_port('port1', neutron_net['id'])
+
+        # Let's force disabling port security for the LSP
+        self.nb_api.lsp_set_port_security(neutron_port['id'], []).execute(
+            check_error=True)
+
+        ovn_port = self.nb_api.db_find(
+            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
+            check_error=True)[0]
+
+        # Assert that port security is now disabled but the 'unknown'
+        # is not set in the addresses column
+        self.assertFalse(ovn_port['port_security'])
+        self.assertNotIn('unknown', ovn_port['addresses'])
+
+        # Call the maintenance task to fix the problem. Note that
+        # NeverAgain is raised so it only runs once at start up
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_for_port_security_unknown_address)
+
+        ovn_port = self.nb_api.db_find(
+            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
+            check_error=True)[0]
+
+        # Assert that 'unknown' was set in the addresses column for
+        # the port
+        self.assertFalse(ovn_port['port_security'])
+        self.assertIn('unknown', ovn_port['addresses'])
+
+        # Now the other way around, let's set port_security in the OVN
+        # table while the 'unknown' address is set in the addresses column
+        self.nb_api.lsp_set_port_security(
+            neutron_port['id'], ovn_port['addresses']).execute(
+            check_error=True)
+
+        ovn_port = self.nb_api.db_find(
+            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
+            check_error=True)[0]
+
+        self.assertTrue(ovn_port['port_security'])
+        self.assertIn('unknown', ovn_port['addresses'])
+
+        # Call the maintenance task to fix the problem. Note that
+        # NeverAgain is raised so it only runs once at start up
+        self.assertRaises(periodics.NeverAgain,
+                          self.maint.check_for_port_security_unknown_address)
+
+        ovn_port = self.nb_api.db_find(
+            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
+            check_error=True)[0]
+
+        # Assert that 'unknown' was removed from the addresses column
+        # for the port
+        self.assertTrue(ovn_port['port_security'])
+        self.assertNotIn('unknown', ovn_port['addresses'])
